import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useUsMap } from './useUsMap';
import { useCities } from './useCities';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';
import {
  schemeYlGnBu,
  csv,
  scaleSqrt,
  scaleOrdinal,
  max,
  format,
  extent,
  scaleLinear
} from 'd3';

const width = 960;
const height = 500;
const margin = { top: 20, right: 250, bottom: 65, left: 90 };
const xAxisLabelOffset = 45;
const yAxisLabelOffset = 20;
const fadeOpacity = 0.2;
const circleRadius = 6;
const legendOffset = 185;

const App = () => {
  const usMap = useUsMap();
  let cities = useCities();
  const [hoveredValue, setHoveredValue] = useState(null);

  if (!usMap || !cities) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xValue = d => d.Longitude;
  const yValue = d => d.Latitude;

  const colorValue = d => d.YEAR;
  const colorLegendLabel = 'YEAR';
  const colorScale = scaleOrdinal()
    .domain(cities.map(colorValue))
    .range(schemeYlGnBu[6]);

  const filteredData = cities.filter(d => hoveredValue === colorValue(d));

  const xScale = scaleLinear()
    .domain(extent(cities, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(cities, yValue))
    .range([0, innerHeight]);

  return (
    <svg width={width} height={height}>
      <g transform={`translate(${innerWidth + legendOffset}, 60)`}>
        <text
          x={xAxisLabelOffset}
          y={-yAxisLabelOffset}
          className="axis-label"
          textAnchor="middle"
        >
          {colorLegendLabel}
        </text>
        <ColorLegend
          tickSpacing={22}
          tickSize={10}
          tickTextOffset={12}
          tickSize={circleRadius}
          colorScale={colorScale}
          onHover={setHoveredValue}
          hoveredValue={hoveredValue}
          fadeOpacity={fadeOpacity}
        />
      </g>

      <g transform={`translate(-70, 0)`}>
        <g opacity={hoveredValue ? fadeOpacity : 1}>
          <Marks
            usMap={usMap}
            cities={cities}
            colorScale={colorScale}
            colorValue={colorValue}
          />
        </g>
        <Marks
          usMap={usMap}
          cities={filteredData}
          colorScale={colorScale}
          colorValue={colorValue}
        />
      </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
